var TMQZ = TMQZ || {};

/**
 * TypeToAnswerQuiz::constructor()
 * Inherit Quiz class
 */
TMQZ.TypeToAnswerQuiz = function( quizContainer ) {
	TMQZ.Quiz.call( this, quizContainer );

	this.score = 0;
	this.answeredCountMax = 20;
};

// Extend the Quiz class
TMQZ.TypeToAnswerQuiz.prototype = Object.create( TMQZ.Quiz.prototype );
TMQZ.TypeToAnswerQuiz.prototype.constructor = TMQZ.TypeToAnswerQuiz;
TMQZ.TypeToAnswerQuiz.prototype.parent = TMQZ.Quiz.prototype;

/**
 * TMQZ.TypeToAnswerQuiz::assignClicks()
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.assignClicks = function( currentQuestion ) {
	var self = this;

	var currentQuestion = $( currentQuestion );
	var $checkButton = currentQuestion.find( '.tmqz-button-check-your-answer' );
	var $showCorrectAnswer = currentQuestion.find( '.tmqz-show-correct-answer' );
	var $clearAnswer = currentQuestion.find( '.clear-answer' );
	var $nextQuestion = currentQuestion.find( '.tmqz-button-next-question' );
	var $answerTextBox = currentQuestion.find( '.tmqz-answer-text-box' );

	// set initial answered-count value
	$answerTextBox.data('answered-count', 0);

	// Create click events
	$checkButton.on( 'click', function(e) {
		e.stopPropagation();
		self.checkAnswer( currentQuestion, this );
	});

	$showCorrectAnswer.on( 'click', function(e) {
		e.stopPropagation();
		self.showCorrectAnswer( currentQuestion );
		// Send track event
		self.ctoTrackUserAction( currentQuestion, 'Show Answer');
	});

	$clearAnswer.on( 'click', function() {
		self.clearAnswer( currentQuestion );
		$answerTextBox.focus();
	});

	$nextQuestion.on( 'click', function(e) {
		self.nextQuestion();
	});

	$answerTextBox.on( 'focus', function(e) {
		self.bindKeyupWhileTyping( currentQuestion );
		self.clearNotification( currentQuestion );
	});

	// unbind kepup on blur to make sure we're not sending multiple events for each answer
	$answerTextBox.on( 'blur', function() {
		$(this).off( 'keyup.check' );
	});

	currentQuestion.on('click', '.tmqz-interactive-content.incorrect .tmqz-text-answers', function(e){
		$answerTextBox.focus();
	});
};

/**
 * TMQZ.TypeToAnswerQuiz::unbindClicks()
 * Unbind click events from question
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.unbindClicks = function( currentQuestion ) {
	// Unbind click events
	currentQuestion.find( '.tmqz-button-check-your-answer' ).off( 'click' );
	currentQuestion.find( '.tmqz-show-correct-answer' ).off( 'click' );
	currentQuestion.find( '.clear-answer' ).off( 'click' );
	currentQuestion.find( '.tmqz-button-next-question' ).off( 'click' );
	currentQuestion.find( '.tmqz-answer-text-box' ).off( 'focus' );
	currentQuestion.find( '.tmqz-answer-text-box' ).off( 'keyup.check' );
	currentQuestion.off( 'click' );
};


/**
 * TMQZ.TypeToAnswerQuiz::activateQuestion()
 * Show a question
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.activateQuestion = function( currentQuestion ) {
  this.assignClicks( currentQuestion );
  currentQuestion.show();

  // if not a touch device, auto focus on the input
  var touch = this.testTouch();
  if( !touch && currentQuestion.find('input[type="text"]' ).length > 0) {
     currentQuestion.find('input[type="text"]')[0].focus();
  }
};

/**
 * TMQZ.TypeToAnswerQuiz::deactivateQuestion()
 * Hide and reset a question
 * @param currentQuestion
 */

TMQZ.TypeToAnswerQuiz.prototype.deactivateQuestion = function( currentQuestion ) {
	currentQuestion.hide();
	this.unbindClicks( currentQuestion );
	this.clearNotification( currentQuestion );
};


/**
 * TMQZ.TypeToAnswerQuiz::checkAnswer()
 * Check the answer and respond with correct/incorrect/show-answer
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.checkAnswer = function( currentQuestion, target ) {
		/* RegEx to remove punctuation from strings. */
		var rePunctuation = /[\u2000-\u206F\u2E00-\u2E7F\\'!"#$%&()*+,\-.\/:;<=>?@\[\]^_`{|}~]/g;

		var $totalScoreEl = currentQuestion.closest('.tm-quiz-container').find( '.tmqz-total-score' );
		var $answerTextBox = currentQuestion.find( '.tmqz-answer-text-box' );
		var enteredAnswer = $answerTextBox.val().trim().toLowerCase().replace( rePunctuation, '' );
		var answers = currentQuestion.find( '.tmqz-text-answers' ).data( 'text-answers' );
		var answeredCount = $answerTextBox.data('answered-count');
		var correct = false;

		for ( var prop in answers ) {
			/* Filter out punctuation from answer list. */
			var filteredAnswers = answers[ prop ].answer.trim().toLowerCase().replace( rePunctuation, '' );

			if ( enteredAnswer == filteredAnswers ) {
				correct = true;
				break;
			}
		}

		// if entered answer is correct
		if ( correct ) {
			// Send track event
			this.ctoTrackUserAction( currentQuestion, enteredAnswer);
			// display "correct" messaging
			this.notifyCorrect( currentQuestion );
			// Update total score
			this.score++;
			$totalScoreEl.text( this.score );
			// unbind click event since can't check anymore
			$(this).off('click');
		} else {
			this.notifyIncorrect( currentQuestion );
			answeredCount++;
			// Send track event
			this.ctoTrackUserAction( currentQuestion, 'Wrong Answer');
			// if hit the limit, kick them to the next question
			if ( answeredCount >= this.answeredCountMax ) {
				this.showCorrectAnswer( currentQuestion );
				// unbind click event since can't check anymore
				$(this).off('click');
			} else {
				// if still have tries left, store the current value
				$answerTextBox.data( 'answered-count', answeredCount );
			}
		}
};

TMQZ.TypeToAnswerQuiz.prototype.ctoTrackUserAction = function( currentQuestion, eventDescription ) {
	var quizName = $(this.quizContainer).data( 'quiz-title' );
	var questionText = currentQuestion.data('question');

	// Send track event
	if (window.cto) {
		cto.trackEvent({
			engagementMeta: quizName + '|' + questionText + '|' + eventDescription
		});
	}
};

/**
 * TMQZ.TypeToAnswerQuiz::clearAnswer()
 * Clear out any values inside the question's text box
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.clearAnswer = function( currentQuestion ) {
  currentQuestion.find( '.tmqz-answer-text-box' ).val('');
  this.clearNotification( currentQuestion );
  this.clearTextBoxClose( currentQuestion );
};


/**
 * TMQZ.TypeToAnswerQuiz::clearTextBoxClose()
 * Hide/disable the "close" X that clears the text box
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.clearTextBoxClose = function( currentQuestion ) {
	currentQuestion.find( '.tmqz-interactive-content' ).addClass('empty');
};

/**
 * TMQZ.TypeToAnswerQuiz::revealNextButton()
 * Show the "next question" button
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.revealNextButton = function( currentQuestion ) {
	currentQuestion.find( '.tmqz-button-next-question' ).addClass('visible').focus();
};


/**
 * TMQZ.TypeToAnswerQuiz::notifyCorrect()
 * Activate the "correct" response state
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.notifyCorrect = function( currentQuestion ) {
	currentQuestion.find( '.tmqz-text-answers' ).addClass('visible');
	currentQuestion.find( '.tmqz-interactive-content' ).removeClass('incorrect').addClass('correct empty');
	this.revealNextButton( currentQuestion );
};

/**
 * TMQZ.TypeToAnswerQuiz::notifyIncorrect()
 * Activate the "incorrect" response state
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.notifyIncorrect = function( currentQuestion ) {
	currentQuestion.find( '.tmqz-text-answers' ).addClass('visible');
	currentQuestion.find( '.tmqz-interactive-content' ).addClass('incorrect');
	currentQuestion.find( '.clear-answer' ).focus();
};


/**
 * TMQZ.TypeToAnswerQuiz::clearNotification()
 * Clear the notification state (either correct or incorrect)
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.clearNotification = function( currentQuestion ) {
	currentQuestion.find( '.tmqz-interactive-content' ).removeClass('correct incorrect reveal-correct');
	currentQuestion.find( '.tmqz-text-answers' ).removeClass('visible');
};


/**
 * TMQZ.TypeToAnswerQuiz::showCorrectAnswer()
 * Show the correct answer (and forfeit additional guesses)
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.showCorrectAnswer = function( currentQuestion ) {
	this.clearNotification( currentQuestion );
	currentQuestion.find( '.tmqz-text-answers' ).addClass( ' visible' );
	currentQuestion.find( '.tmqz-interactive-content' ).addClass('reveal-correct');
	currentQuestion.find( '.tmqz-button-next-question' ).focus();
};


/**
 * TMQZ.TypeToAnswerQuiz::bindKeyupWhileTyping()
 * Bind keyup events to allow for various visual effects and UX interactions while typing
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.bindKeyupWhileTyping = function( currentQuestion ) {
	var self = this;

	currentQuestion.find('.tmqz-answer-text-box').on( 'keyup.check', function(e) {
		e.stopPropagation();

		// if hit the enter key and there is text present, submit the answer
		if( (e.keyCode === 13) && ($(this).val().length > 0 )) {
			$(e.currentTarget).closest( '.tmqz-question' ).find( '.tmqz-button-check-your-answer' ).click();
		}

		// if hit the escape key, remove focus from the text box to disable event listeners
		else if(e.keyCode === 27) {
			$(this).blur();
		}

		// if neither of those special keys are hit and characters have been input, show the "clear answer" X
		else if( $(this).val().length > 0 ) {
			$(this).closest('.tmqz-interactive-content').removeClass('empty');
		}

		// if they now have no characters (potentially after delete/backspace) clear the X
		else {
			self.clearTextBoxClose( currentQuestion );
		}
	});
};


/**
 * TMQZ.TypeToAnswerQuiz::getResult()
 * Get the Quiz result
 * @param currentQuestion
 */
TMQZ.TypeToAnswerQuiz.prototype.getResult = function() {
	var quizObj = this;

	for ( var i = 0; i < quizObj.results.length; i++ ) {
		var result = quizObj.results[ i ];

		if ( quizObj.score >= $( result ).data( 'points-from' ) && quizObj.score <= $( result ).data( 'points-to' ) ) {
			var resultText = $( result ).find( '.tmqz-result-text' );
			resultText.html( quizObj.replaceTokens( resultText.html() ) );
			quizObj.assignSharingClicks( result );

			// bind the retry event to restart without refreshing the page
			quizObj.bindRetry( result );
			return result;
		}
	}
};
